# -*- coding: utf-8 -*-
# Generated by Django 1.10.2 on 2016-11-09 15:11
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0006_auto_20161121_0241'),
    ]

    operations = [
        migrations.AddField(
            model_name='enterprisecustomer',
            name='catalog',
            field=models.PositiveIntegerField(help_text='Course catalog for the Enterprise Customer.', null=True),
        ),
        migrations.AddField(
            model_name='historicalenterprisecustomer',
            name='catalog',
            field=models.PositiveIntegerField(help_text='Course catalog for the Enterprise Customer.', null=True),
        ),
    ]
