# -*- coding: utf-8 -*-
# Generated by Django 1.11.15 on 2018-10-25 11:59
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0055_auto_20181015_1112'),
    ]

    operations = [
        migrations.AddField(
            model_name='enterprisecustomerreportingconfiguration',
            name='pgp_encryption_key',
            field=models.TextField(blank=True, help_text='The key for encryption, if PGP encrypted file is required.', null=True, verbose_name='PGP Encryption Key'),
        ),
    ]
